package com.xy.service;

import cn.easyes.core.biz.EsPageInfo;
import cn.easyes.core.conditions.LambdaEsQueryWrapper;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.dto.BackMqttDto;
import com.xy.dto.MqttDto;
import com.xy.dto.PaterDto;
import com.xy.entity.MqttCommand;
import com.xy.mapper.MqttCommandMapper;
import com.xy.producer.MqttProducer;
import com.xy.service.MqttService;
import com.xy.utils.*;
import com.xy.utils.enums.MqttCommandStatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.List;

import static com.xy.utils.EsBeans.toPageBean;

@Slf4j
@Service
@Api(tags = "mqtt接口")
@AllArgsConstructor
public class MqttServiceImpl implements MqttService {

    private MqttProducer mqttProducer;

    private MqttCommandMapper mqttCommandMapper;

    @Override
    @ApiOperation("指令发布")
    public List<MqttDto.Vo> sendMqtt(List<MqttDto> mqttDtos) {
        List<MqttDto.Vo> list = new ArrayList<>(mqttDtos.size());
        mqttDtos.forEach(mqttDto -> {
            //给予默认值
            PaterDto pater = mqttDto.getPater()
                    .setSn(String.valueOf(YitIdHelper.nextId()))
                    .setTime(DataTime.getSring());
            //延迟发布处理
            String delayTime = mqttDto.getDelayTime();
            String topic = mqttDto.getTopic();
            if (!StringUtils.isEmpty(delayTime)) {
                //消息消费时间 - 发布时间的差值 = 距今被消费的秒数
                long m = DataTime.diff(DataTime.toLocal(pater.getTime()), DataTime.toLocal(delayTime), "s");
                topic = String.format("$delayed/%d/%s", m, mqttDto.getTopic());
                mqttDto.setTimeout(mqttDto.getTimeout() + (int) m);
            }
            //处理消息过期时间
            pater.setTimeout(DataTime.getStringAround(0, 0, 0, 0, 0, mqttDto.getTimeout()));
            //翻译枚举
            JSONObject value = JSONUtil.parseObj(pater)
                    .set(LambdaUtils.getProperty(PaterDto::getType1), pater.getType1().getKey())
                    .set(LambdaUtils.getProperty(PaterDto::getType2), pater.getType2().getKey());
            String wkSn = pater.getWkSn();
            try {
                //发送消息
                mqttProducer.sendToMqtt(value.toString(), topic, mqttDto.getLevel());
                //指令记录
                saveMqttCommand(mqttDto, value, pater.getAck() ? MqttCommandStatusEnum.COMMAND_SEND : MqttCommandStatusEnum.COMMAND_SUCCESS, null);
                list.add(new MqttDto.Vo().setWkSn(wkSn).setResult(true));
            } catch (Exception e) {
                //指令记录
                saveMqttCommand(mqttDto, value, MqttCommandStatusEnum.COMMAND_SEND_FAIL, e.getMessage());
                list.add(new MqttDto.Vo().setWkSn(wkSn).setResult(false));
            }
        });
        return list;
    }

    @Override
    @ApiOperation("指令结果回执")
    public R mqttBack(BackMqttDto backMqttDto) {
        MqttCommand mqttCommand = mqttCommandMapper.selectById(backMqttDto.getSn());
        if (!Emptys.check(mqttCommand)) {
            return R.fail("数据不存在");
        }
        //重复上报，前面的时间不能覆盖后面的时间
        if (!StringUtils.isEmpty(mqttCommand.getExecTime())) {
            if (DataTime.stringContrast(DataTime.toString(backMqttDto.getExecTime()), mqttCommand.getExecTime()) < 0) {
                return R.ok();
            }
        }
        mqttCommand.setStatus(Integer.parseInt(backMqttDto.getBool() ? MqttCommandStatusEnum.COMMAND_SUCCESS.getKey() : MqttCommandStatusEnum.COMMAND_EXC_FAIL.getKey()))
                .setExecTime(DataTime.toString(backMqttDto.getExecTime()));
        mqttCommandMapper.updateById(mqttCommand);
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<MqttDto.Vo2>> page(@RequestBody MqttDto.Page page) {
        LambdaEsQueryWrapper<MqttCommand> lambdaEsQueryWrapper = new EsQueryWrapper().eqWrapper(page, MqttCommand.class)
                .like(MqttCommand::getError)
                .like(MqttCommand::getValue)
                .ge(MqttCommand::getSendTime, page.getBeginSendTime())
                .le(MqttCommand::getSendTime, page.getEndSendTime())
                .ge(MqttCommand::getTimeout, page.getBeginTimeout())
                .le(MqttCommand::getTimeout, page.getEndTimeout())
                .ge(MqttCommand::getDelayTime, page.getBeginDelayTime())
                .le(MqttCommand::getDelayTime, page.getEndDelayTime())
                .build()
                .orderByDesc(MqttCommand::getSendTime);
        PageBean pageBean = page.getPage();
        EsPageInfo<MqttCommand> mqttCommandEsPageInfo = mqttCommandMapper.pageQuery(lambdaEsQueryWrapper, (int) pageBean.getCurrent(), (int) pageBean.getSize());
        return R.ok(toPageBean(MqttDto.Vo2.class, mqttCommandEsPageInfo));
    }
    

    /**
     * 新增指令记录
     *
     * @param mqttDto 参数对象
     * @param value   参数json对象
     * @param status  状态
     * @param error   错误信息
     */
    private void saveMqttCommand(MqttDto mqttDto, JSONObject value, MqttCommandStatusEnum status, String error) {
        PaterDto pater = mqttDto.getPater();
        MqttCommand mqttCommand = Beans.copy(Beans.copy(MqttCommand.class, mqttDto), pater)
                .setId(pater.getSn())
                .setType1(pater.getType1().getKey())
                .setType2(pater.getType2().getKey())
                .setStatus(Integer.parseInt(status.getKey()))
                .setValue(value.toString())
                .setSendTime(pater.getTime())
                .setExecTime(status.getKey().equals(MqttCommandStatusEnum.COMMAND_SUCCESS.getKey()) ? DataTime.getSring() : null)
                .setNum(1)
                .setError(error);
        mqttCommandMapper.insert(mqttCommand);
    }
}
